#!/usr/bin/env python

import sys
import json

from attrs import define, field
import cattrs
import attrs
from enum import Enum
import typer
from typing_extensions import Annotated


# @unique
class MatchType(Enum):
    BEGIN = "begin"
    MATCH = "match"
    END = "end"


@define
class Path:
    text: str = field(
        validator=attrs.validators.instance_of(str)
    )


@define
class Line:
    text: str = field(
        validator=attrs.validators.instance_of(str)
    )


@define
class Submatch:
    start: int = field(
        validator=attrs.validators.instance_of(int)
    )


@define
class Match:
    path: Path = field(
        validator=attrs.validators.instance_of(Path)
    )
    lines: Line = field(
        validator=attrs.validators.instance_of(Line)
    )
    line_number: int = field(
        validator=attrs.validators.instance_of(int)
    )
    submatches: list[Submatch] = field(
    )


@define
class RGLine:
    type: MatchType = field(
        validator=attrs.validators.instance_of(MatchType)
    )
    data: Match = field(
        validator=attrs.validators.instance_of(Match)
    )

    def row(self):
        return self.data.line_number

    def file(self):
        return self.data.path.text

    def text(self):
        return self.data.lines.text.rstrip()

    def column(self):
        return self.data.submatches[0].start + 1


def main(
        rows_before: Annotated[int, typer.Option("--rb")],
        lines_count: Annotated[int, typer.Option("--lc")],
):
    lines = sys.stdin.readlines()
    if len(lines) == 0:
        sys.exit(0)
    result = []
    for line in lines:
        value = json.loads(line)
        try:
            self = cattrs.structure(value, RGLine)
            # TODO: remove magic numbers
            row_start = max(self.row() - rows_before, 1)
            result.append(f"{self.file()}:{self.row()}:{self.column()}:{row_start}:{lines_count}:{self.text()}")
        except Exception as e:
            pass

    print("\n".join(result))


if __name__ == "__main__":
    typer.run(main)



import subprocess
import os

from pathlib import Path

from decouple import config as decouple_config
from decouple import Config, RepositoryEnv

import cattrs

from dirs_navigator import *

if os.environ.get("CONFIG_PATH"):
    config = Config(RepositoryEnv(os.environ["CONFIG_PATH"]))
elif Path(".env.local").is_file():
    config = Config(RepositoryEnv(".env.local"))
else:
    config = decouple_config


def sel_env2():
    import glob

    options = glob.glob(f"{os.environ['HOME']}/.navigator/*.yaml")
    values = {}
    for o in options:
        i = o.rfind("/")
        values[o[i + 1:-5]] = o

    print(values)

    return 0

def sel_env():
    import glob

    options = glob.glob(f"{os.environ['HOME']}/.navigator/*.yaml")
    values = {}
    for o in options:
        i = o.rfind("/")
        values[o[i + 1:-5]] = o
    my_env = os.environ.copy()
    my_env["GUM_FILTER_PLACEHOLDER"] = f"Choose an environment:"
    result = subprocess.run(
        ["gum", "filter"], input="\n".join(values.keys()), stdout=subprocess.PIPE, text=True, env=my_env
    )
    open(f"{os.environ['HOME']}/.navigator/.environment", "w").write(result.stdout.strip())
    return 0


def choose_destination():
    from yaml import load, dump
    try:
        from yaml import CLoader as Loader, CDumper as Dumper
    except ImportError:
        from yaml import Loader, Dumper

    environment = open(f"{os.environ['HOME']}/.navigator/.environment").read()
    doc = load(open(f"{os.environ['HOME']}/.navigator/{environment}.yaml"), Loader=Loader)
    all_paths = cattrs.structure(doc, list[Project])
    values = {p.name: p.rootPath for p in all_paths}
    my_env = os.environ.copy()
    my_env["GUM_FILTER_PLACEHOLDER"] = f"Choose an environment:"
    result = subprocess.run(
        ["gum", "filter"], input="\n".join(values.keys()), stdout=subprocess.PIPE, text=True, env=my_env
    )
    sel = result.stdout.strip()
    if sel:
        selected_path = values[sel]
        if selected_path.startswith("~"):
            selected_path = selected_path.replace("~", os.environ["HOME"])
        print(selected_path)
        return 0
    else:
        return 1



#!/usr/bin/env python

import sys

def main():
    lines = sys.stdin.readlines()
    if len(lines) == 0:
        sys.exit(0)
    line = lines[0]
    result = line.split(":")
    file = result[0]
    line = result[1]
    column = result[2].strip()

    print(f""" +"call cursor({line}, {column})" "{file}" """)
